/**
 *
 * @file tests/plugin/Plugin.cpp
 *
 * @copyright 2008-2024 Bordeaux INP, CNRS (LaBRI UMR 5800), Inria,
 *                      Univ. Bordeaux. All rights reserved.
 *
 * @author Camille Ordronneau
 * @author Mathieu Faverge
 *
 * @date 2024-07-17
 */
#include <QWidget>
#include "Plugin.hpp"
// #include "Plug3.hpp"

Plugin *Plugin::new_instance(const std::string &name) {
    Plugin *plugin = NULL;
//     if(name == "plug3") {
//         plugin = new Plug3();
//     }
    return plugin;
}
